FetchInstructionResult

Enum FetchInstructionResult 

pub enum FetchInstructionResult<I, Peeked = I>
where I: Instruction,
{ Instruction(Peeked), Continue, Break, Err(ExecutionError<<<I as Instruction>::Reg as Register>::Type>), }
Result of InstructionFetcher::fetch_instruction() and InstructionFetcher::peek_instruction() calls.

Peeked is the instruction itself when fetching, and whatever InstructionFetcher::Peeked is when peeking.

Variants



Instruction(Peeked)

Instruction to execute



Continue

Nothing to execute here, carry on from wherever the program counter now points



Break

Stop execution



Err(ExecutionError<<<I as Instruction>::Reg as Register>::Type>)

Fetching failed